@@ -83,16 +83,17 @@
 			Returns this link value based on it's expression and type
 		*/
 		function getLinkValue($oCol_) {
-			$sVal="";
+			$sVal		= "";
+			$oError	= new PHPReportsErrorTr();
 			if($this->_sType=="STATIC")
 				$sVal=$this->_sExpr;
 			else if($this->_sType=="DYNAMIC") {
 				if(!isset($oCol_))
-					new PHPReportsError("Specified a dynamic link but no COLUMN element found","contactdeveloper.php?msg=Specified a dynamic link but no COLUMN element found.");
+					$oError->showMsg("DYNLINK");
 				$sVal=$oCol_->getValue($this->_sExpr);
 			}else if($this->_sType=="EXPRESSION") {						
 				if(!isset($oCol_))
-					new PHPReportsError("Specified an expression link but no COLUMN element found","contactdeveloper.php?msg=Specified an expression link but no COLUMN element found.");
+					$oError->showMsg("EXPLINK");
 				$sVal = htmlspecialchars($oCol_->availExpr($this->_sExpr));
 			}
 			return $sVal;
