@@ -42,8 +42,14 @@
 		var $_sClassName;		// report class name to create
 		var $_sTmp;				// temporary dir
 		var $_oCon;				// database connection handle
+		var $_oQuery;			// executed query
 		var $_iPageSize;		// page size
 		var $_aBench;			// benchmark registers
+		var $_sLang;			// language
+		var $_bBody;			// no HTML BODY shown on the report
+		var $_bDeleteXML;		// if needs to delete the XML file after using it
+		var $_oError;
+		var $_oInput;			// input plugins
 
 		/***************************************************************************
 		*																									*
@@ -75,6 +81,11 @@
 			$this->_sClassName		= "PHPReport";
 			$this->_iPageSize			= 0;
 			$this->_aBench				= Array();
+			$this->_sLang				= "default";
+			$this->_bBody				= true;
+			$this->_bDeleteXML		= false;
+			$this->_oError				= new PHPReportsErrorTr();
+			$this->_oInput				= Array();
 
 			/*
 				Now we get the XSLT processor
@@ -84,14 +95,125 @@
 			$oProcFactory = new XSLTProcessorFactory();
 			$this->_oProc = $oProcFactory->get();
 			if(is_null($this->_oProc))
-				new PHPReportsError("There is no XSLT processor available.<br/>".
-										  "Check if you compiled with Sablotron (PHP4) with --enable-xslt or<br/>".
-										  "PHP5 XSL support, with --enable-xsl, correctly.","noxsltprocessor.php");
+				$this->_oError->showMsg("NOXSLT");
 				
 			// check path stuff
 			if(is_null(getPHPReportsFilePath()))
-				new PHPReportsError("Seems that you didn't the PHPReports path on the PHP include_path.<br/>".
-				                    "I don't know where the classes are.","nopath.php");
+				$this->_oError->showMsg("NOPATH");
+		}
+
+		/******************************************************************************
+		*																										*
+		*	Create a quick report from a layout file and some parameters.					*
+		*																										*
+		******************************************************************************/
+		function createFromTemplate($sTitle="NO DEFINED TITLE",$sFile=null,$oParms=null,$oDocument=null,$oGroups=null){
+			$sPath	= getPHPReportsFilePath();
+			$oError	= new PHPReportsErrorTr();
+			$sIf		= $this->getDatabaseInterface();
+			$sFile	= $sFile ? $sFile : realpath($sPath."/template.xml");
+
+			// if the file does not exist
+			if(!file_exists($sFile))
+				$oError->showMsg("NOTEMPLATE",$sFile);
+
+			// get the template contents
+			$sFileContents = file_get_contents($sFile);
+
+			// check if there are some parameters here ...
+			if($oParms){
+				$sParms = null;
+				if(is_object($oParms))	$sParms = $oParms->write();
+				if(is_string($oParms))	$sParms = $oParms;
+				if($sParms)
+					$sFileContents = str_replace("<REPLACE_WITH_PARAMETERS/>",$sParms,$sFileContents);
+			}
+
+			// if no groups info was specified, follow the default behaviour: all the fields from the query,
+			// with no group break
+			if(!$oGroups){
+				// include the database interface and try to open the connection and execute the query	
+				$sIfFile = realpath($sPath."/database/db_".$sIf.".php");
+				if(!file_exists($sIfFile))
+					$oError->showMsg("NOIF",$sIf);
+				include_once $sIfFile; 
+
+				// if the database connection is null, open it
+				if(is_null($this->_oCon))
+					$this->_oCon = @PHPReportsDBI::db_connect(Array($this->sUser,$this->sPass,$this->sCon,$this->sDatabase)) or $oError->showMsg("REFUSEDCON");
+
+				// if there are some input filters ...
+				if($this->_oInput){
+					foreach($this->_oInput as $oFilter){
+						$oFilter->setConnection($this->_oCon);
+						$oFilter->setSQL(trim($this->sSQL));
+						$this->sSQL = trim($oFilter->run());
+					}
+					// there is no need to run the filters again
+					$this->_oInput = null;
+				}
+
+				// run the query
+				$this->_oQuery = @PHPReportsDBI::db_query($this->_oCon,trim($this->sSQL)) or $oError->showMsg("QUERYERROR");
+
+				// insert the column names
+				$sNames				= "";
+				$sReplacedNames	= "";
+				$sTotals				= "";
+
+				$iColNum = PHPREportsDBI::db_colnum($this->_oQuery);
+				for($i=1; $i<=$iColNum; $i++){
+					$sName				= PHPReportsDBI::db_columnName($this->_oQuery,$i);
+					$sExtra				= isNumericType(PHPReportsDBI::db_columnType($this->_oQuery,$i))?" ALIGN=\"RIGHT\"":"";
+					$sReplacedNames  .= "<COL CELLCLASS=\"bold\"$sExtra>".ucfirst(strtolower(str_replace("_"," ",$sName)))."</COL>";
+					$sNames			  .= "<COL TYPE=\"FIELD\"$sExtra>$sName</COL>";
+					
+					if(isNumericType(PHPReportsDBI::db_columnType($this->_oQuery,$i)))
+						$sTotals		  .= "\t\t\t<COL TYPE=\"EXPRESSION\" ALIGN=\"RIGHT\">\$this->getSum(\"$sName\");</COL>\n";
+					else
+						$sTotals		  .= "\t\t\t<COL></COL>\n";
+				}
+
+				// build the group info
+				$sGroup			= "<GROUP REPRINT_HEADER_ON_PAGEBREAK='TRUE'><HEADER><ROW><REPLACE_WITH_REPLACED_COLUMN_NAMES/></ROW></HEADER><FIELDS><ROW><REPLACE_WITH_COLUMN_NAMES/></ROW></FIELDS></GROUP>";
+				$sGroup			= str_replace("<REPLACE_WITH_REPLACED_COLUMN_NAMES/>",$sReplacedNames,$sGroup);
+				$sGroup			= str_replace("<REPLACE_WITH_COLUMN_NAMES/>",$sNames,$sGroup);
+				$sFileContents = str_replace("<REPLACE_WITH_GROUP_INFO/>",$sGroup,$sFileContents);
+
+				// if no document info is set, make one
+				if(!$oDocument)
+					$oDocument = "<DOCUMENT>\n\t<FOOTER>\n\t\t<ROW>\n$sTotals\t\t</ROW>\t</FOOTER>\n</DOCUMENT>";
+			}else{
+				$sGroups = null;
+				if(is_object($oGroups))	$sGroups	= $oGroups->write();
+				if(is_string($oGroups))	$sGroups = $oGroups;
+				if($sGroups)
+					$sFileContents = str_replace("<REPLACE_WITH_GROUP_INFO/>",$sGroups,$sFileContents);
+			}
+
+			// check if there is some document info
+			if($oDocument){
+				$sDocument = null;
+				if(is_object($oDocument))	$sDocument = $oDocument->write();
+				if(is_string($oDocument))	$sDocument = $oDocument;
+				if($sDocument)
+					$sFileContents = str_replace("<REPLACE_WITH_DOCUMENT_INFO/>",$sDocument,$sFileContents);
+			}
+
+			// replace the report title
+			$sFileContents = str_replace("<REPLACE_WITH_TITLE/>",$sTitle,$sFileContents);
+			// print htmlspecialchars($sFileContents);
+
+			// create the temporary XML file
+			$sTemp = tempnam($this->_sTmp,"tempphprpt");
+
+			// this is just for PHP4 compability
+			$fHand = fopen($sTemp,"w");
+			fwrite($fHand,$sFileContents);
+			fclose($fHand);
+
+			$this->_bDeleteXML	= true;		// flag to delete the temporary file
+			$this->sXML				= $sTemp;	// the XML layout file is the temporary file now
 		}
 
 		/******************************************************************************
@@ -114,10 +236,11 @@
 			$aParm["nodatamsg"]	= $this->_sNoDataMsg;	// no data msg
 			$aParm["nodatafunc"] = $this->_sNoDataFunc;	// no data function
 			$aParm["pagesize"]	= $this->_iPageSize>0?$this->_iPageSize:"";							
+			$aParm["language"]	= $this->_sLang;
 
 			// create the parameters keys array - with element numbers or element keys
 			$aKeys = null;
-			if(!is_null($this->_oParm)){
+			if(is_array($this->_oParm)){
 				$aKeys = array_keys($this->_oParm);
 				$iSize = sizeof($this->_oParm);
 				for($i=0; $i<$iSize; $i++){
@@ -160,11 +283,8 @@
 				eval($sRst);
 			else {
 			// include the generated classes, if it was created
-				if(!file_exists($sOut)){
-					new PHPReportsError("PHPReports could not create the output code to make your report ".
-											  "runs. Please check if the web server user have rights to write ".
-											  "in your ".dirname($sOut)." directory. Script aborted.");
-				}
+				if(!file_exists($sOut))
+					$this->_oError->showMsg("NOCODE",array($sOut));
 				require_once($sOut);	
 			}	
 			$this->_aBench["code_eval_end"] = time();
@@ -174,17 +294,16 @@
 			
 			// set the database connection handle, if there is one
 			$oReport->setDatabaseConnection($this->_oCon);
+			$oReport->setInputFilters($this->_oInput);
+			$oReport->setQuery($this->_oQuery);
 
 			// run the generated class
 			$this->_sXMLOutputFile = $oReport->run($this->_sXMLOutputFile,$this->_aEnv);
 			$this->_aBench = array_merge($this->_aBench,$oReport->getBenchmarks());
 
 			// check if the XML file exists, we need data!
-			if(!file_exists($this->_sXMLOutputFile)){
-				new PHPReportsError("PHPReports could not find the XML file with your data (".$this->_sXMLOutputFile.") to make your report ".
-										  "runs. Please check if the web server user have rights to write ".
-										  "in your temporary directory. Script aborted.");
-			}
+			if(!file_exists($this->_sXMLOutputFile))
+				$this->_oError->showMsg("NOXML",array($this->_sXMLOutputFile));
 
 			/*
 				Now we have a XML file with the report contents ... what to to with it???
@@ -197,9 +316,11 @@
 				$oOut = $this->createOutputPlugin("default");
 				$oOut->setInput ($this->_sXMLOutputFile);
 				$oOut->setOutput($this->sOut);
+				$oOut->setBody($this->_bBody);
 				$this->setOutputPlugin($oOut);
 			}else{
 				$oOut = $this->_oOutputPlugin;
+				$oOut->setBody($this->_bBody);
 				$oOut->setInput($this->_sXMLOutputFile);
 				if(!is_null($this->sOut))
 					$oOut->setOutput($this->sOut);
@@ -214,6 +335,10 @@
 			$this->_aBench["output_end"] = time();
 			$this->_aBench["report_start"] = $iReportStart;
 			$this->_aBench["report_end"] = time();		
+
+			// if needs to delete the XML file
+			if($this->_bDeleteXML)
+				unlink($this->sXML);
 			return $this->_sXMLOutputFile;
 		}
 
@@ -244,7 +369,7 @@
 		******************************************************************************/		
 		function setXML($sXML_) {
 			if(!file_exists($sXML_))
-				new PHPReportsError("File $sXML_ was not found.");
+				$this->_oError->showMsg("NOXMLSET",array($sXML_));
 			$this->sXML = $sXML_;
 		}
 
@@ -266,7 +391,7 @@
 		******************************************************************************/		
 		function setXSLT($sXSLT_) {
 			if(!file_exists($sXSLT_))
-				new PHPReportsError("File $sXSLT_ was not found.");
+				$this->_oError->showMsg("NOXSLTSET",array($sXSLT_));
 			$this->sXSLT = $sXSLT_;
 		}
 
@@ -501,13 +626,27 @@
 			
 			// check if the required plugin exists
 			if(!file_exists($sFullPath))
-				new PHPReportsError("There is no $sName_ output plugin ($sFullPath)");
+				$this->_oError->showMsg("NOPLUGIN",array($sName_,$sFullPath));
 			include $sFullPath; 
 			
 			$oOut = new PHPReportOutput($this->sXML);
 			return $oOut;
 		}
 
+		function addInputPlugin($sName_,$oGroupDesc_,$sGroupKey_,$oOptions_=null){
+			$sName  = ucwords(strtolower($sName_));
+			$sClass = "PHPReportInput$sName";
+			$sFullPath = $this->_sPath."/input/PHPReportInput$sName.php";
+			
+			// check if the required plugin exists
+			if(!file_exists($sFullPath))
+				$this->_oError->showMsg("NOPLUGIN",array($sName_,$sFullPath));
+			include $sFullPath; 
+
+			$oIn = new $sClass($oGroupDesc_,$sGroupKey_,$oOptions_);
+			array_push($this->_oInput,$oIn);
+		}
+
 		/******************************************************************************
 		*																										*
 		*	Output plugin for the final format														*
@@ -660,5 +799,77 @@
 		function setDatabaseConnection(&$_oCon){
 			$this->_oCon =& $_oCon;
 		}		
+
+		/******************************************************************************
+		*																										*
+		*	Set a valid query	made before																* 
+		*																										*
+		******************************************************************************/
+		function setQuery($oQuery_){
+			$this->_oQuery=$oQuery_;
+		}
+
+		/******************************************************************************
+		*																										*
+		*	Here's the deal: if the user have a session opened, the language will be	*
+		*	stored there. If more than one user is using the system with different		*
+		*	languages, each one will see the specified language (no way to run two		*
+		*	reports with different languages for each user). If no session is opened,	*
+		*	the language value is already there on GLOBALS, so we can retrieve from		*
+		*	there, but this will allow just one language for all the reports.				*
+		*																										*
+		******************************************************************************/
+		function setLanguage($sLang_="default"){
+			$this->_sLang=$sLang_;
+			$_SESSION["phpReportsLanguage"] = $sLang_;
+			$GLOBALS["phpReportsLanguage"] = $sLang_;
+		}
+
+		function getLanguage(){
+			return $this->_sLang;
+		}
+
+		function setBody($b=true){
+			$this->_bBody=$b;
+		}
+
+		function getBody(){
+			return $this->_bBody;
+		}
+	}
+
+	class PHPReportTemplateElement {
+		var $_aAttrs;
+		var $_aChildren;
+		var $_sType;
+
+		function PHPReportTemplateElement($sType=null,$aAttrs=null){
+			$this->_sType		= $sType;
+			$this->_aChildren	= array();
+			$this->_aAttrs		= $aAttrs ? $aAttrs : array();
+		}
+
+		function addAttr($sKey,$sValue){
+			$this->_aAttrs[$sKey]=$sValue;
+		}
+
+		function addChild($oChild){
+			array_push($this->_aChildren,$oChild);
+		}
+
+		function write(){
+			if($this->_sType){
+				$sStr = "<".$this->_sType;
+				foreach($this->_aAttrs as $sKey=>$sValue)
+					$sStr .= strtoupper($sKey)=="VALUE"?"":" $sKey=\"$sValue\" ";
+				$sStr .= ">";
+			}
+			$sStr .= $this->_aAttrs["VALUE"];
+			foreach($this->_aChildren as $oChild)
+				$sStr .= $oChild->write();
+			if($this->_sType)
+				$sStr .= "</".$this->_sType.">";
+			return $sStr;
+		}
 	}
 ?>
